/* 
 *
 * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. The Blender
 * Foundation also sells licenses for use in proprietary software under
 * the Blender License.  See http://www.blender.org/BL/ for information
 * about this.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
 * All rights reserved.
 *
 * This is a new part of Blender.
 *
 * Contributor(s): Willian P. Germano
 *
 * ***** END GPL/BL DUAL LICENSE BLOCK *****
*/

#include "Scene.h"

#include "Camera.h"
#include "DNA_scene_types.h"
#include "BKE_scene.h"
#include "BIF_drawscene.h"
#include "BSE_headerbuttons.h"

/* Define imported from the old BPY_macros.h */
#define getUsers(x) ((x)->id.us)
#define INVALID_FRAME -99999

PyObject *Scene_createPyObject (Scene *scene);
PyObject *M_ObjectCreatePyObject (struct Object *obj);
PyObject *Camera_createPyObject (Camera *cam);


/*****************************************************************************/
/* Function:              M_Scene_New                                         */
/* Python equivalent:     Blender.Scene.New                                   */
/*****************************************************************************/
static PyObject *M_Scene_New(PyObject *self, PyObject *args)
{
  char*        name_str="Scene";
  C_Scene      *py_scene; /* for Scene Data object wrapper in Python */
  Scene        *bl_scene; /* for actual Scene Data we create in Blender */

  printf ("In Scene_New()\n");

  if (!PyArg_ParseTuple(args, "|s", &name_str))
    return (EXPP_ReturnPyObjError (PyExc_AttributeError,
            "expected string(name) or empty argument"));

  bl_scene = add_scene(name_str); /* first create in Blender */
  if (bl_scene) /* now create the wrapper obj in Python */
    py_scene = (C_Scene*)Scene_createPyObject(bl_scene);
  else
    return (EXPP_ReturnPyObjError (PyExc_RuntimeError,
                            "couldn't create Scene Data in Blender"));

  if (py_scene == NULL)
    return (EXPP_ReturnPyObjError (PyExc_MemoryError,
                            "couldn't create Scene Data object"));

  py_scene->scene = bl_scene; /* link Python scene wrapper with Blender Scene */

  return (PyObject *)py_scene;
}

/*****************************************************************************/
/* Function:              M_Scene_Get                                         */
/* Python equivalent:     Blender.Scene.Get                                   */
/* Description:           Receives a string and returns the scene data obj    */
/*                        whose name matches the string.  If no argument is  */
/*                        passed in, a list of all scene data names in the    */
/*                        current scene is returned.                         */
/*****************************************************************************/
static PyObject *M_Scene_Get(PyObject *self, PyObject *args)
{
  char *name = NULL;
  Scene *scene_iter;

  if (!PyArg_ParseTuple(args, "|s", &name))
    return (EXPP_ReturnPyObjError (PyExc_TypeError,
            "expected string argument (or nothing)"));

  scene_iter = G.main->scene.first;
  
  if (name) { /* (name) - Search scene by name */
    
    C_Scene *wanted_scene = NULL;
    
    while ((scene_iter) && (wanted_scene == NULL))
      {
	
	if (strcmp (name, scene_iter->id.name+2) == 0)
	  wanted_scene = (C_Scene *)Scene_createPyObject(scene_iter);
	
	scene_iter = scene_iter->id.next;
      }
    
    if (wanted_scene == NULL) { /* Requested scene doesn't exist */
      char error_msg[64];
      PyOS_snprintf(error_msg, sizeof(error_msg),
		    "Scene \"%s\" not found", name);
      return (EXPP_ReturnPyObjError (PyExc_NameError, error_msg));
    }
    
    return (PyObject *)wanted_scene;
  }
  
  else { /* () - return a list of all scenes in the scene */
    int index = 0;
    PyObject *scenelist, *pystr;
    
    scenelist = PyList_New (BLI_countlist (&(G.main->scene)));
    
    if (scenelist == NULL)
      return (PythonReturnErrorObject (PyExc_MemoryError,
				       "couldn't create PyList"));
    
    while (scene_iter) {
      pystr = PyString_FromString (scene_iter->id.name+2);
      
      if (!pystr)
	return (PythonReturnErrorObject (PyExc_MemoryError,
					 "couldn't create PyString"));
      
      PyList_SET_ITEM (scenelist, index, pystr);
      
      scene_iter = scene_iter->id.next;
      index++;
    }
    
    return (scenelist);
  }
}



static PyObject *M_Scene_getCurrent(PyObject *self, PyObject *args)
{
  return (C_Scene *)Scene_createPyObject(G.scene);
}


/*****************************************************************************/
/* Function:              M_Scene_Init                                        */
/*****************************************************************************/
/* Needed by the Blender module, to register the Blender.Scene submodule */
PyObject *M_Scene_Init (void)
{
  PyObject  *submodule;

  submodule = Py_InitModule3("Blender.Scene", M_Scene_methods, M_Scene_doc);

  return (submodule);
}

/* Three Python Scene_Type helper functions needed by the Object module: */

/*****************************************************************************/
/* Function:    Scene_createPyObject                                          */
/* Description: This function will create a new C_Scene from an existing      */
/*              Blender scene structure.                                      */
/*****************************************************************************/
PyObject *Scene_createPyObject (Scene *scene)
{
  C_Scene *pyscene;
  
  pyscene = (C_Scene *)PyObject_NEW (C_Scene, &Scene_Type);
  
  if (!pyscene)
    return EXPP_ReturnPyObjError (PyExc_MemoryError,
				  "couldn't create C_Scene object");
  
  pyscene->scene = scene;
      
  return (PyObject *)pyscene;
}

/*****************************************************************************/
/* Function:    Scene_checkPyObject                                           */
/* Description: This function returns true when the given PyObject is of the */
/*              type Scene. Otherwise it will return false.                   */
/*****************************************************************************/
int Scene_checkPyObject (PyObject *pyobj)
{
  return (pyobj->ob_type == &Scene_Type);
}

/*****************************************************************************/
/* Function:    Scene_fromPyObject                                            */
/* Description: This function returns the Blender scene from the given        */
/*              PyObject.                                                    */
/*****************************************************************************/
Scene *Scene_fromPyObject (PyObject *pyobj)
{
  return ((C_Scene *)pyobj)->scene;
}

/*****************************************************************************/
/* Python C_Scene methods:                                                    */
/*****************************************************************************/
static PyObject *Scene_getName(C_Scene *self)
{
  PyObject *attr = PyString_FromString(self->scene->id.name+2);

  if (attr) return attr;

  return (EXPP_ReturnPyObjError (PyExc_RuntimeError,
          "couldn't get Scene.name attribute"));
}

static PyObject *Scene_getUsers(C_Scene *self)
{ 
  PyObject *attr = PyInt_FromLong(getUsers(self->scene));

  if (attr) return attr;

  return (EXPP_ReturnPyObjError (PyExc_RuntimeError,
          "couldn't get Scene.users attribute"));
}

static PyObject *Scene_getBlockType(C_Scene *self)
{
  PyObject *attr = Py_BuildValue("s", "Scene");

  if (attr) return attr;

  return (EXPP_ReturnPyObjError (PyExc_RuntimeError,
          "couldn't get Scene.block_type attribute"));
}


static PyObject *Scene_setName(C_Scene *self, PyObject *args)
{
  char *name = NULL;
  char buf[21];

  if (!PyArg_ParseTuple(args, "s", &name))
    return (EXPP_ReturnPyObjError (PyExc_TypeError,
            "expected string argument"));
  
  PyOS_snprintf(buf, sizeof(buf), "%s", name);

  rename_id(&self->scene->id, buf);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *Scene_copy(C_Scene *self, PyObject *args)
{
  int dup_objects = 0;

  if (!PyArg_ParseTuple(args, "|i", &dup_objects))
    return (EXPP_ReturnPyObjError (PyExc_TypeError,
            "expected integer argument (or nothing)"));

  if (!self->scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));

  return Scene_createPyObject(copy_scene(self->scene, dup_objects));
}

static PyObject *Scene_update(C_Scene *self, PyObject *args)
{

  Scene* scene = Scene_fromPyObject((PyObject*)self);
  
  if (!scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));

  if (!PyArg_ParseTuple(args, ""))
    return (EXPP_ReturnPyObjError (PyExc_TypeError,
            "no argument expected"));

  sort_baselist(scene);

  Py_INCREF(Py_None);
  return Py_None;	
}

static PyObject *Scene_link(C_Scene *self, PyObject *args)
{

  /*  DataBlock *block;
  Object *object;
  Scene* scene = Scene_fromPyObject((PyObject*)self);
  
  if (!scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));

  if (!PyArg_ParseTuple(args, "O!", &DataBlock_Type, &block))
    return (EXPP_ReturnPyObjError (PyExc_TypeError,
            "DataBlock object expected as argument." ));
    
  if (DataBlock_type(block) != ID_OB)
    EXPP_ReturnPyObjError(PyExc_TypeError, "link: invalid Object type");

  object = PYBLOCK_AS_OBJECT(block);

  if (!scene_linkObject(scene, object))
    EXPP_ReturnPyObjError(PyExc_RuntimeError, "Object already in scene!");
  */ 
  Py_INCREF(Py_None);
  return Py_None;	
}

static PyObject *Scene_unlink(C_Scene *self, PyObject *args)
{
  /*
  PyObject *retval;
  DataBlock *block;
  Object *object;
  Scene* scene = Scene_fromPyObject((PyObject*)self);
  
  if (!scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));

  if (!PyArg_ParseTuple(args, "O!", &DataBlock_Type, &block))
    return (EXPP_ReturnPyObjError (PyExc_TypeError,
            "DataBlock object expected as argument." ));
    
  if (DataBlock_type(block) != ID_OB)
    EXPP_ReturnPyObjError(PyExc_TypeError, "link: invalid Object type");

  object = PYBLOCK_AS_OBJECT(block);

  if (!scene_unlinkObject(scene, object))
    retval = Py_BuildValue("i", 0);
  else	
    retval = Py_BuildValue("i", 1);
  
  Py_INCREF(retval);
  return retval;
  */
  Py_INCREF(Py_None);
  return Py_None;	
}

static PyObject *Scene_getChildren(C_Scene *self, PyObject *args)
{

  Scene* scene = Scene_fromPyObject((PyObject*)self);
  PyObject *pylist= PyList_New(0);
  PyObject *b;
  Base* base = scene->base.first;
  
  if (!scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));

  if (!PyArg_ParseTuple(args, ""))
    EXPP_ReturnPyObjError (PyExc_TypeError,
            "no argument expected." );
    
  while (base)
    {	
      //     b = (PyObject *) DataBlock_fromData(base->object);	
      b = M_ObjectCreatePyObject (base->object);
      PyList_Append(pylist, b);	
      Py_XDECREF(b);             // because PyList_Append increfs!
      base = base->next;
    }	

  return pylist;
}

static PyObject *Scene_getCurrentCamera(C_Scene *self, PyObject *args)
{

  Scene* scene = Scene_fromPyObject((PyObject*)self);
  Object* object=NULL;
  
  if (!scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));

  if (!PyArg_ParseTuple(args, ""))
    EXPP_ReturnPyObjError (PyExc_TypeError,
            "no argument expected." );
    
  object = scene->camera;
  if (!object) {
    Py_INCREF(Py_None);	
    return Py_None;
  }

  return Camera_createPyObject((Camera*)object->data);
}

static PyObject *Scene_setCurrentCamera(C_Scene *self, PyObject *args)
{

  /*
  Scene* scene = Scene_fromPyObject((PyObject*)self);
  Object* object=NULL;
  C_Camera* block;
  
  if (!scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));

  if (!PyArg_ParseTuple(args, "O!", &Camera_Type, &block))
    EXPP_ReturnPyObjError (PyExc_TypeError,
            "Object argument expected." );

  if (!DataBlock_isType(block, ID_OB)) 
    EXPP_ReturnPyObjError (PyExc_TypeError,
            "Object type expected." );
  
  
  scene->camera = PYBLOCK_AS_OBJECT(block);
  if (scene_getCurrent() == scene) // update current scene display
    window_update_curCamera(object);

  */
  
  Py_INCREF(Py_None);	
  return Py_None;
  
}

static PyObject *Scene_getRenderdir(C_Scene *self, PyObject *args)
{
  
  Scene* scene = Scene_fromPyObject((PyObject*)self);

  if (!scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));
  
  if (!PyArg_ParseTuple(args, ""))
    EXPP_ReturnPyObjError (PyExc_TypeError,
            "no argument expected." );
    
  return PyString_FromString(scene->r.pic);
  
}

static PyObject *Scene_getBackbufdir(C_Scene *self, PyObject *args)
{
  
  Scene* scene = Scene_fromPyObject((PyObject*)self);

  if (!scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));
  
  if (!PyArg_ParseTuple(args, ""))
    EXPP_ReturnPyObjError (PyExc_TypeError,
            "no argument expected." );
    
  return PyString_FromString(scene->r.backbuf);
  
}

static PyObject *Scene_frameSettings(C_Scene *self, PyObject *args)
{
  RenderData *rd = 0;
  int current = INVALID_FRAME;
  int start = INVALID_FRAME;
  int end = INVALID_FRAME;
  Scene* scene = Scene_fromPyObject((PyObject*)self);
  
  if (!scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));
    
  rd = &scene->r;
  
  if (!PyArg_ParseTuple(args, "|iii", &start, &end, &current))
    EXPP_ReturnPyObjError (PyExc_TypeError,
            "expected 1,2 or 3 integers." );
    
  if (start > 0) {
    rd->sfra = start;
  }	
  if (end > 0) {
    rd->efra = end;
  }	
  if (current > 0) {
    rd->cfra = current;
  }	
  return Py_BuildValue("(iii)", rd->sfra, rd->efra, rd->cfra);
}

static PyObject *Scene_makeCurrent(C_Scene *self, PyObject *args)
{
  Scene* scene = Scene_fromPyObject((PyObject*)self);

  if (!scene) (EXPP_ReturnPyObjError (PyExc_RuntimeError,"couldn't operate on a NULL scene"));
  
  if (!PyArg_ParseTuple(args, ""))
    EXPP_ReturnPyObjError (PyExc_TypeError,"no argument expected." );
  
  set_scene(scene);
  
  Py_INCREF(Py_None);	
  return Py_None;
}


/*****************************************************************************/
/* Function:    SceneDeAlloc                                                  */
/* Description: This is a callback function for the C_Scene type. It is       */
/*              the destructor function.                                     */
/*****************************************************************************/
static void SceneDeAlloc (C_Scene *self)
{
  PyObject_DEL (self);
}

/*****************************************************************************/
/* Function:    SceneGetAttr                                                  */
/* Description: This is a callback function for the C_Scene type. It is       */
/*              the function that accesses C_Scene member variables and       */
/*              methods.                                                     */
/*****************************************************************************/
static PyObject *SceneGetAttr (C_Scene *self, char *name)
{
  PyObject *attr = Py_None;

  if (strcmp(name, "name") == 0)
    attr = Scene_getName(self);
  else if (strcmp(name, "users") == 0)
    attr = Scene_getUsers(self);
  else if (strcmp(name, "block_type") == 0)
    attr = Scene_getBlockType(self);

  if (!attr)
    return (EXPP_ReturnPyObjError (PyExc_MemoryError,
                      "couldn't create PyObject"));

  if (attr != Py_None) return attr; /* member attribute found, return it */

  /* not an attribute, search the methods table */
  return Py_FindMethod(C_Scene_methods, (PyObject *)self, name);
}

/*****************************************************************************/
/* Function:    SceneSetAttr                                                  */
/* Description: This is a callback function for the C_Scene type. It is the   */
/*              function that changes Scene Data members values. If this      */
/*              data is linked to a Blender Scene, it also gets updated.      */
/*****************************************************************************/
static int SceneSetAttr (C_Scene *self, char *name, PyObject *value)
{
  PyObject *valtuple; 
  PyObject *error = NULL;

  valtuple = Py_BuildValue("(N)", value); /* the set* functions expect a tuple */

  if (!valtuple)
    return EXPP_ReturnIntError(PyExc_MemoryError,
                  "SceneSetAttr: couldn't create tuple");

  if (strcmp (name, "name") == 0)
    error = Scene_setName (self, valtuple);

  else /* ... or no member with the given name was found */
    return (EXPP_ReturnIntError (PyExc_AttributeError,
				 "attribute not found"));
  
  Py_DECREF(valtuple);
  
  if (error != Py_None) return -1;

  Py_DECREF(Py_None); /* was incref'ed by the called Scene_set* function */
  return 0; /* normal exit */
}

/*****************************************************************************/
/* Function:    SceneCompare                                                  */
/* Description: This is a callback function for the C_Scene type. It          */
/*              compares two Scene_Type objects. Only the "==" and "!="       */
/*              comparisons are meaninful. Returns 0 for equality and -1 if  */
/*              they don't point to the same Blender Scene struct.            */
/*              In Python it becomes 1 if they are equal, 0 otherwise.       */
/*****************************************************************************/
static int SceneCompare (C_Scene *a, C_Scene *b)
{
	Scene *pa = a->scene, *pb = b->scene;
	return (pa == pb) ? 0:-1;
}

/*****************************************************************************/
/* Function:    ScenePrint                                                    */
/* Description: This is a callback function for the C_Scene type. It          */
/*              builds a meaninful string to 'print' scene objects.           */
/*****************************************************************************/
static int ScenePrint(C_Scene *self, FILE *fp, int flags)
{ 
  fprintf(fp, "[Scene \"%s\"]", self->scene->id.name+2);
  return 0;
}

/*****************************************************************************/
/* Function:    SceneRepr                                                     */
/* Description: This is a callback function for the C_Scene type. It          */
/*              builds a meaninful string to represent scene objects.         */
/*****************************************************************************/
static PyObject *SceneRepr (C_Scene *self)
{
  return PyString_FromString(self->scene->id.name+2);
}
