/* 
 *
 * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. The Blender
 * Foundation also sells licenses for use in proprietary software under
 * the Blender License.  See http://www.blender.org/BL/ for information
 * about this.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
 * All rights reserved.
 *
 * This is a new part of Blender.
 *
 * Contributor(s): Willian P. Germano
 *
 * ***** END GPL/BL DUAL LICENSE BLOCK *****
*/

#ifndef EXPP_SCENE_H
#define EXPP_SCENE_H

#include <Python.h>

#include <BKE_main.h>
#include <BKE_global.h>
#include <BKE_object.h>
#include <BKE_library.h>
#include <BLI_blenlib.h>
#include <DNA_scene_types.h>

#include "constant.h"
#include "gen_utils.h"

/*****************************************************************************/
/* Python API function prototypes for the Scene module.                     */
/*****************************************************************************/
static PyObject *M_Scene_New (PyObject *self, PyObject *args);
static PyObject *M_Scene_Get (PyObject *self, PyObject *args);
static PyObject *M_Scene_getCurrent (PyObject *self, PyObject *args);

/*****************************************************************************/
/* The following string definitions are used for documentation strings.      */
/* In Python these will be written to the console when doing a               */
/* Blender.Scene.__doc__                                                    */
/*****************************************************************************/
char M_Scene_doc[] =
"The Blender Scene module.";

char M_Scene_New_doc[] = " <not done> ";

char M_Scene_Get_doc[] =
"(name) - return the scene with the name 'name', \
returns None if not found.\n If 'name' is not specified, \
it returns a list of all scenes.";

static char M_Scene_getCurrent_doc[] =
"() - returns currently active Scene";

/*****************************************************************************/
/* Python method structure definition for Blender.Scene module:             */
/*****************************************************************************/
struct PyMethodDef M_Scene_methods[] = {
  {"New",         M_Scene_New,         METH_VARARGS, M_Scene_New_doc},
  {"Get",         M_Scene_Get,         METH_VARARGS, M_Scene_Get_doc},
  {"get",         M_Scene_Get,         METH_VARARGS, M_Scene_Get_doc},
  {"getCurrent",  M_Scene_getCurrent,  METH_VARARGS, M_Scene_getCurrent_doc},
  {NULL, NULL, 0, NULL}
};

/*****************************************************************************/
/* Python C_Scene structure definition:                                     */
/*****************************************************************************/
typedef struct {
  PyObject_HEAD
  Scene *scene;
} C_Scene;

/*****************************************************************************/
/* Python C_Scene methods declarations:                                     */
/*****************************************************************************/
static PyObject *Scene_getName(C_Scene *self);
static PyObject *Scene_getUsers(C_Scene *self);
static PyObject *Scene_getBlockType(C_Scene *self);
static PyObject *Scene_setName(C_Scene *self, PyObject *args);
static PyObject *Scene_copy(C_Scene *self, PyObject *args);
static PyObject *Scene_link(C_Scene *self, PyObject *args);
static PyObject *Scene_unlink(C_Scene *self, PyObject *args);
static PyObject *Scene_getChildren(C_Scene *self, PyObject *args);
static PyObject *Scene_getCurrentCamera(C_Scene *self, PyObject *args);
static PyObject *Scene_setCurrentCamera(C_Scene *self, PyObject *args);
static PyObject *Scene_getRenderdir(C_Scene *self, PyObject *args);
static PyObject *Scene_getBackbufdir(C_Scene *self, PyObject *args);
static PyObject *Scene_frameSettings(C_Scene *self, PyObject *args);
static PyObject *Scene_makeCurrent(C_Scene *self, PyObject *args);
static PyObject *Scene_update(C_Scene *self, PyObject *args);

/*****************************************************************************/
/* Python C_Scene methods table:                                            */
/*****************************************************************************/
static PyMethodDef C_Scene_methods[] = {
 /* name, method, flags, doc */
  {"getName", (PyCFunction)Scene_getName, METH_NOARGS, "() - Return Scene Data name"},
  {"getUsers", (PyCFunction)Scene_getUsers, METH_NOARGS, "() - Return Scene number of users "},
  {"getBlockType", (PyCFunction)Scene_getBlockType, METH_NOARGS, "() - Return Scene block type"},
  {"setName", (PyCFunction)Scene_setName, METH_VARARGS, "(str) - Change Scene Data name"},
  {"copy", (PyCFunction)Scene_copy, METH_VARARGS, "<undocummented>"},
  {"link", (PyCFunction)Scene_link, METH_VARARGS, "<undocummented>"},
  {"unlink", (PyCFunction)Scene_unlink, METH_VARARGS, "<undocummented>"}, 
  {"getChildren", (PyCFunction)Scene_getChildren, METH_VARARGS, "<undocummented>"},
  {"getCurrentCamera", (PyCFunction)Scene_getCurrentCamera, METH_VARARGS, "<undocummented>"},
  {"setCurrentCamera", (PyCFunction)Scene_setCurrentCamera, METH_VARARGS, "<undocummented>"},
  {"getRenderdir", (PyCFunction)Scene_getRenderdir, METH_VARARGS, "<undocummented>"},
  {"getBackbufdir", (PyCFunction)Scene_getBackbufdir, METH_VARARGS, "<undocummented>"},
  {"frameSettings", (PyCFunction)Scene_frameSettings, METH_VARARGS, "<undocummented>"},
  {"makeCurrent", (PyCFunction)Scene_makeCurrent, METH_VARARGS, "<undocummented>"},
  {"update", (PyCFunction)Scene_update, METH_VARARGS, "<undocummented>"},
  {0,NULL,0,0}
};

/*****************************************************************************/
/* Python Scene_Type callback function prototypes:                          */
/*****************************************************************************/
static void SceneDeAlloc (C_Scene *self);
static int ScenePrint (C_Scene *self, FILE *fp, int flags);
static int SceneSetAttr (C_Scene *self, char *name, PyObject *v);
static PyObject *SceneGetAttr (C_Scene *self, char *name);
static int SceneCompare (C_Scene *a, C_Scene *b);
static PyObject *SceneRepr (C_Scene *self);

/*****************************************************************************/
/* Python Scene_Type helper functions needed by Blender (the Init function) */
/* and Object modules.                                                       */
/*****************************************************************************/
PyObject *M_Scene_Init (void);
PyObject *SceneCreatePyObject (Scene *cam);
Scene *SceneFromPyObject (PyObject *pyobj);
int SceneCheckPyObject (PyObject *pyobj);

/*****************************************************************************/
/* Python Scene_Type structure definition:                                  */
/*****************************************************************************/
PyTypeObject Scene_Type =
{
  PyObject_HEAD_INIT(&PyType_Type)
  0,                                      /* ob_size */
  "Scene",                               /* tp_name */
  sizeof (C_Scene),                      /* tp_basicsize */
  0,                                      /* tp_itemsize */
  /* methods */
  (destructor)SceneDeAlloc,              /* tp_dealloc */
  (printfunc)ScenePrint,                 /* tp_print */
  (getattrfunc)SceneGetAttr,             /* tp_getattr */
  (setattrfunc)SceneSetAttr,             /* tp_setattr */
  (cmpfunc)SceneCompare,                 /* tp_compare */
  (reprfunc)SceneRepr,                   /* tp_repr */
  0,                                      /* tp_as_number */
  0,                                      /* tp_as_sequence */
  0,                                      /* tp_as_mapping */
  0,                                      /* tp_as_hash */
  0,0,0,0,0,0,
  0,                                      /* tp_doc */ 
  0,0,0,0,0,0,
  C_Scene_methods,                       /* tp_methods */
  0,                                      /* tp_members */
};

#endif /* EXPP_SCENE_H */
